import requests


class Stats:
    def __init__(self, stats_config):
        self.stats = stats_config

    def update(self,value):
        for stat in self.stats:
            stat.update(value)

    def update_server(self,stat):
        stat_dict = {}
        stat_dict['id'] = stat.id
        stat_dict['value'] = stat.value
        response = requests.post('http://' + '192.168.3.132' + ':8000/channelstats/', json=stat_dict)

    def update_on_server(self):
        for stat in self.stats:
            if stat.update_on_server:
                self.update_server(stat)
                stat.value_updated_on_server()



from .channel_interface import ChannelInterface
import logging
from OpenHub.globals import id_hardware_map
import time

class ModProbeTemp(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, config, hardware_serial_no=None, serial_no=None, *args, **kwargs):
        self.mod_probe = id_hardware_map[config['hardware']]
        self.serial_no = config['id']
        self.type = config['type']
        self.device_file = self.mod_probe.device_file
        super().__init__(config=config,hardware_serial_no=self.mod_probe.serial_no, serial_no=self.serial_no, *args, **kwargs)

    async def get_raw_data(self):
        return {'value':self.mod_probe.read_temp_c()}



from adafruit_mcp3xxx.analog_in import AnalogIn
from .channel_interface import ChannelInterface
import logging
import uuid
from OpenHub.globals import id_hardware_map


class MCP3008Analog(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, mcp=None, channel_index=None, hardware_serial_no=None, serial_no=uuid.uuid4(),
                 *args, **kwargs):
        super.__init__(hardware_serial_no)
        self.type = __name__
        self.analog_in = AnalogIn(mcp, channel_index)
        super().__init__(hardware_serial_no=hardware_serial_no, serial_no=serial_no, *args, **kwargs)

    def get_raw_data(self):
        return self.analog_in.voltage



import logging
from .data_transformer import DataTransformer


class Product(DataTransformer):

    def perform_op(self, inputs):
        product = 1
        for input in inputs:
            product = product * input
        return product


from .stat_interface import StatInterface

class Max(StatInterface):

    def update(self, value):
        if value > self.value:
            self.value = value
            self.check_if_value_updated()



import json

from .channel_interface import ChannelInterface
import logging
from OpenHub.globals import id_hardware_map


class PiPicoAnalog(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, config, hardware_serial_no=None, serial_no=None, *args, **kwargs):
        self.pipico = id_hardware_map[config['hardware']]
        self.channel_index = config['channel_index']
        self.serial_no = config['id']
        self.type = config['type']
        super().__init__(config=config,hardware_serial_no=self.pipico.serial_no, serial_no=self.serial_no, *args, **kwargs)

    async def get_raw_data(self):
        command = "data"
        channel = str(self.channel_index)
        try:
            response = await self.pipico.send_command(command,channel)
            parsed_response = response.decode('utf-8').replace("'", '"')
            sensor_response = json.loads(parsed_response)
        except Exception as e:
            self.logger.log(level=logging.WARN,msg=str(e))
            return {}
        return sensor_response



from .channel_interface import ChannelInterface
import logging


class PMSAAirQuality25(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, pmsa_airquality=None, hardware_serial_no=None, serial_no=None, *args, **kwargs):
        self.pmsa_airquality = pmsa_airquality
        self.type = __name__
        super().__init__(config=config,hardware_serial_no=hardware_serial_no, serial_no=serial_no, *args, **kwargs)

    def get_raw_data(self):
        try:
            aqdata = self.pmsa_airquality.read()
            pm25 = aqdata["pm25 standard"]
            return pm25
        except RuntimeError:
            print("Unable to read from sensor, retrying...")
            return -1



import logging
from .data_transformer import DataTransformer


class Divide(DataTransformer):

    def perform_op(self, inputs):
        if len(inputs) == 2:
            return inputs[0]/inputs[1]
        else:
            return 0


import logging
from serial import Serial

from hardware_interfaces.base_io.base_io_interface import BaseIOInterface


class SerialIO(BaseIOInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, port, *args, **kwargs):
        self.serial_port = Serial(port.device, 9600, timeout=1)
        super(SerialIO, self).__init__(*args, **kwargs)





from abc import ABC, abstractmethod

class StatInterface(ABC):
    def __init__(self, dct):
        if 'id' in dct:
            self.id = dct['id']
        if 'type' in dct:
            self.type = dct['type']
        if 'value' in dct:
            self.old_value = dct['value']
            self.value = dct['value']

        self.update_on_server = False

    @abstractmethod
    def update(self, value):
        pass

    def check_if_value_updated(self):
        if self.old_value != self.value:
            self.update_on_server = True

    def value_updated_on_server(self):
        self.old_value = self.value
        self.update_on_server = False;


from OpenHub.homekit_accessories.homkit_sensor_interface import HomeKitSensorInterface
import logging

class ETapeSensor(HomeKitSensorInterface):
    logger = logging.getLogger(__name__)
    run_debug_message = "Current ETape Level: "


    def __init__(self, serial_no=None, display_name=None, channel_interface_serial_no=None, config=None, *args, **kwargs):
        self.scale = float((1 / 65536) * 100)
        super().__init__(serial_no=serial_no, display_name=display_name,
                         channel_interface_serial_no=channel_interface_serial_no, config=config, *args, **kwargs)

    def set_display_name(self, display_name):
        if display_name is None:
            return "ETape"
        else:
            return display_name

    def add_functional_service(self):
        return self.add_preload_service("PressureSensor")

    def add_functional_service_characteristic(self):
        return self.service.get_characteristic('Pressure')




import json
from OpenHub.hardware_interfaces.channels.stat.max import Max
from OpenHub.hardware_interfaces.channels.stat.min import Min



class StatDecoder(json.JSONDecoder):
    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, dct):
        type = dct['type']
        if type == 'MAX':
            return Max(dct)
        if type == 'MIN':
            return Min(dct)



import logging
from abc import ABC, abstractmethod
import json
from OpenHub.globals import id_channels_map,id_stats_map

class DataTransformer(ABC):
    logger = logging.getLogger(__name__)

    homekit_accessory_serial_no = None

    def __init__(self, dct, data_transformers=None):
        self.channels = []
        self.stats=dct['channel_stats']
        self.constants = dct['data_transformer_constants']

        for channel in dct['channels']:
            self.channels.append(id_channels_map[channel])
        for stat in dct['channel_stats']:
            self.stats.append(id_stats_map[str(stat['id'])])
        self.data_transformers = dct['data_transformers']

        super().__init__()

    async def run(self):
        outputs = []
        if self.data_transformers is not None:
            for transformer in self.data_transformers:
                outputs.append(await transformer.run())
        if self.channels is not None:
            for channel in self.channels:
                outputs.append(await channel.run())
        if self.stats is not None:
            for stat in self.stats:
                outputs.append(stat.value)

        return self.perform_op(outputs)

    @abstractmethod
    def perform_op(self, inputs):
        pass



import json
from OpenHub.hardware_interfaces.channels.stat.json.stat_decoder import StatDecoder
from OpenHub.hardware_interfaces.channels.stats.stats import Stats


class StatsDecoder(json.JSONDecoder):
    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, dct):
        stats = []
        for stat in dct:
            stat = json.loads(stat, cls=StatDecoder)
            stats.append(stat)
        return Stats(stats)




import json

import serial

from hardware_interfaces.pi_pico import PiPico
from Adafruit_GPIO import GPIO
from homekit_accessories.liquid_level_sensor import LiquidLevelSensor
from homekit_accessories.etape import ETapeSensor
from homekit_accessories.pressure_sensor import PressureSensor
from homekit_accessories import Pump


class PiPicoInit():
    interrupts = [2, 3, 17]
    pico_configs = []
    COMs = []
    picos = []
    pico_sensor_serial_no_to_serial_com = {}

    def init_picos(self):
        self.get_pico_ports()
        self.setup_interrupt_to_acm_mapping()


    def get_pico_ports(self):
        pts = list(serial.tools.list_ports.comports())

        for pt in pts:
            if 'ACM' in pt[0]:
                self.COMs.append(pt[0])

    def setup_interrupt_to_acm_mapping(self):

        for interrupt in self.interrupts:
            GPIO.setup(interrupt, GPIO.OUT)
            GPIO.output(interrupt, GPIO.HIGH)

        for ser in self.COMs:
            command = "init" + "\n"
            ser.write(command.encode('utf-8'))
            pico_data = ser.readline()
            sensor_response = pico_data[:-2]
            print(sensor_response)
            pico_config = json.loads(sensor_response)
            pico_config['serial_com'] = ser
            self.pico_configs.append(pico_config)

        for interrupt in self.interrupts:
            GPIO.output(interrupt, GPIO.LOW)

    def initialize_picos(self, sensor_hub):
        for pico_config in self.pico_configs:
            pico = PiPico(pico_config)
            self.picos.append(pico)

    def setup_pico_sensors(self, sensor_hub):
        for pico in self.picos:
            for channel in pico.hardware_id_channels_map:
                type = channel['type']
                sensor = None
                if type == 'optical':
                    sensor = LiquidLevelSensor(pico)
                elif type == 'etape':
                    sensor = ETapeSensor(pico)
                elif type == 'pressure':
                    sensor = PressureSensor(pico)
                elif type == 'pump':
                    sensor = Pump(pico)
                if sensor is not None:
                    sensor_hub.add_accessory(sensor)



from abc import ABC, abstractmethod
import uuid


class ChannelInterface(ABC):
    def __init__(self, config, hardware_serial_no=None, serial_no=uuid.uuid4(), channel_stats=None, **kwargs):
        self.serial_no = serial_no
        self.hardware_serial_no = hardware_serial_no
        self.stats = channel_stats
        if 'keep_statistics' in config:
            self.keep_statistics = config['keep_statistics']
        else:
            self.keep_statistics = False

    def update_stats(self,value):
        if self.keep_statistics:
            self.stats.update(value)
            self.stats.update_on_server()

    async def run(self):
        value = await self.get_raw_data()
        if self.keep_statistics:
            self.update_stats(value)
        return value


    @abstractmethod
    async def get_raw_data(self):
        pass



import json
from OpenHub.homekit_accessories.air_temperature import AirTemperatureSensor
from OpenHub.homekit_accessories.hub import Hub
from OpenHub.homekit_accessories.humidity import HumiditySensor
from OpenHub.homekit_accessories.light_sensor import LightSensor
from OpenHub.homekit_accessories.soil_moisture_sensor import SoilMoistureSensor
from OpenHub.homekit_accessories.soil_temperature_sensor import SoilTemperatureSensor
from OpenHub.homekit_accessories.pressure_sensor import PressureSensor
from OpenHub.homekit_accessories.liquid_level_sensor import LiquidLevelSensor
from OpenHub.homekit_accessories.etape import ETapeSensor
from OpenHub.homekit_accessories.pump import Pump
from OpenHub.homekit_accessories.camera import Camera

from OpenHub.homekit_accessories.relay import Relay

from OpenHub.data_transformers.constants.constant import Constant
from OpenHub.data_transformers.data_transformer import DataTransformer
from OpenHub.data_transformers.difference import Difference
from OpenHub.data_transformers.divide import Divide
from OpenHub.data_transformers.max import Max
from OpenHub.data_transformers.min import Min
from OpenHub.data_transformers.product import Product
from OpenHub.data_transformers.sum import Sum

import logging


class HomekitDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, dct):
        logger = logging.getLogger(HomekitDecoder.__name__)
        logger.info(str(dct))
        type = None
        model = None

        if 'model' in dct.keys():
            model = dct['model']

        if model == 'DataTransformerConstants':
            return Constant(id=dct['id'],index=dct['index'],value=dct['value'])
        elif model == 'DataTransformer':
            type = dct['type']

            if type == 'Divide':
                return Divide(dct)
            if type == 'Max':
                return Max(dct)
            if type == 'Multiply':
                return Product(dct)
            if type == 'Subtract':
                return Difference(dct)
            if type == 'Min':
                return Min(dct)
            if type == 'Sum':
                return Sum(dct)
        else:

            if 'type' in dct.keys():
                type = dct['type']

            if type == AirTemperatureSensor.__name__:
                return AirTemperatureSensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            if type == HumiditySensor.__name__:
                return HumiditySensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            elif type == LightSensor.__name__:
                return LightSensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)

            elif type == SoilMoistureSensor.__name__:
                return SoilMoistureSensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)

            elif type == SoilTemperatureSensor.__name__:
                return SoilTemperatureSensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            elif type == PressureSensor.__name__:
                return PressureSensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            elif type == Pump.__name__:
                return Pump(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            elif type == Relay.__name__:
                return Relay(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            elif type == LiquidLevelSensor.__name__:
                return LiquidLevelSensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            elif type == ETapeSensor.__name__:
                return ETapeSensor(dct['id'], dct['display_name'], dct['channels'][0],config=dct)
            elif type == Camera.__name__:
                return Camera(dct['id'], dct['display_name'])
            else:
                return Hub(dct['id'], dct['display_name'])



import json
from OpenHub.hardware_interfaces.channels.dht22_humidity import DHT22Humidity
from OpenHub.hardware_interfaces.channels.dht22_temp import DHT22Temp
from OpenHub.hardware_interfaces.channels.mcp3008analog import MCP3008Analog
from OpenHub.hardware_interfaces.channels.mod_probe_temp import ModProbeTemp
from OpenHub.hardware_interfaces.channels.pi_pico_analog import PiPicoAnalog
from OpenHub.hardware_interfaces.channels.pi_pico_ac_analog import PiPicoACAnalog
from OpenHub.hardware_interfaces.channels.pi_pico_pump import PiPicoPump
from OpenHub.hardware_interfaces.channels.veml7700_light import VEML7700Light
from OpenHub.hardware_interfaces.channels.veml7700_lux import VEML7700Lux

from OpenHub.hardware_interfaces.channels.pi_pico_relay import PiPicoRelay
from OpenHub.hardware_interfaces.channels.stats.json.stats_decoder import StatsDecoder

class ChannelDecoder(json.JSONDecoder):
    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, config):
        channel_stats=None
        if 'channel_stats' in config.keys() and config['channel_stats'] is not None:
            channel_stats = json.loads(config['channel_stats'], cls=StatsDecoder)

        type = config['type']
        if type == DHT22Humidity.__name__:
            return DHT22Humidity(config=config, channel_stats=channel_stats)
        if type == DHT22Temp.__name__:
            return DHT22Temp(config=config, channel_stats=channel_stats)
        elif type == MCP3008Analog.__name__:
            return MCP3008Analog()

        elif type == ModProbeTemp.__name__:
            return ModProbeTemp(config=config, channel_stats=channel_stats)

        elif type == PiPicoAnalog.__name__:
            return PiPicoAnalog(config=config, channel_stats=channel_stats)
        elif type == PiPicoACAnalog.__name__:
            return PiPicoACAnalog(config=config, channel_stats=channel_stats)
        elif type == PiPicoPump.__name__:
            return PiPicoPump(config=config, channel_stats=channel_stats)

        elif type == PiPicoRelay.__name__:
            return PiPicoRelay(config=config, channel_stats=channel_stats)

        elif type == VEML7700Light.__name__:
            return VEML7700Light(config=config, channel_stats=channel_stats)

        elif type == VEML7700Lux.__name__:
            return VEML7700Lux(config=config, channel_stats=channel_stats)



import logging

from OpenHub import PiPicoGPI
import RPi.GPIO as GPIO


class PiGPI(PiPicoGPI):
    logger = logging.getLogger(__name__)

    def __init__(self, *args, **kwargs):
        super(PiGPI, self).__init__(*args, **kwargs)
        GPIO.setup(self.pin, GPIO.IN)
